@@ -45,7 +45,6 @@ static ssize_t pull_func (gnutls_transport_ptr_t p, void *data, size_t size);
 static const char *human_addr (const struct sockaddr *sa, socklen_t salen,
                                char *buf, size_t buflen);
 static int wait_for_connection (int fd);
-static gnutls_session_t initialize_tls_session (void);
 static int generate_dh_params (void);
 
 /* Use global credentials and parameters to simplify
@@ -173,7 +172,10 @@ main (void)
       else
         continue;
 
-      session = initialize_tls_session ();
+      gnutls_init (&session, GNUTLS_SERVER | GNUTLS_DATAGRAM);
+      gnutls_priority_set (session, priority_cache);
+      gnutls_credentials_set (session, GNUTLS_CRD_CERTIFICATE, x509_cred);
+
       gnutls_dtls_prestate_set (session, &prestate);
       gnutls_dtls_set_mtu (session, mtu);
 
@@ -191,7 +193,10 @@ main (void)
         {
           ret = gnutls_handshake (session);
         }
-      while (ret < 0 && gnutls_error_is_fatal (ret) == 0);
+      while (ret == GNUTLS_E_INTERRUPTED || ret == GNUTLS_E_AGAIN);
+      /* Note that DTLS may also receive GNUTLS_E_LARGE_PACKET.
+       * In that case the MTU should be adjusted.
+       */
 
       if (ret < 0)
         {
@@ -212,17 +217,24 @@ main (void)
             }
           while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
 
-          if (ret < 0)
+          if (ret < 0 && gnutls_error_is_fatal (ret) == 0)
+            {
+              fprintf (stderr, "*** Warning: %s\n", gnutls_strerror (ret));
+              continue;
+            }
+          else if (ret < 0)
             {
               fprintf (stderr, "Error in recv(): %s\n",
                        gnutls_strerror (ret));
               break;
             }
+
           if (ret == 0)
             {
               printf ("EOF\n\n");
               break;
             }
+
           buffer[ret] = 0;
           printf ("received[%.2x%.2x%.2x%.2x%.2x%.2x%.2x%.2x]: %s\n",
                   sequence[0], sequence[1], sequence[2], sequence[3],
@@ -409,24 +421,11 @@ human_addr (const struct sockaddr *sa, socklen_t salen,
   return save_buf;
 }
 
-static gnutls_session_t
-initialize_tls_session (void)
-{
-  gnutls_session_t session;
-
-  gnutls_init (&session, GNUTLS_SERVER | GNUTLS_DATAGRAM);
-
-  gnutls_priority_set (session, priority_cache);
-
-  gnutls_credentials_set (session, GNUTLS_CRD_CERTIFICATE, x509_cred);
-
-  return session;
-}
-
 static int
 generate_dh_params (void)
 {
-  int bits = gnutls_sec_param_to_pk_bits (GNUTLS_PK_DH, GNUTLS_SEC_PARAM_LOW);
+  int bits = 
+    gnutls_sec_param_to_pk_bits (GNUTLS_PK_DH, GNUTLS_SEC_PARAM_LEGACY);
 
   /* Generate Diffie-Hellman parameters - for use with DHE
    * kx algorithms. When short bit length is used, it might
